package com.example.newton2;

import android.graphics.Color;
import android.graphics.Typeface;
import android.os.Bundle;

import androidx.fragment.app.Fragment;

import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

import com.example.newton2.Common.Common;
import com.example.newton2.Interface.IQuestion;
import com.example.newton2.Model.CurrentQuestion;
import com.example.newton2.Model.Question;
import com.squareup.picasso.Callback;
import com.squareup.picasso.Picasso;


/**
 * A simple {@link Fragment} subclass.
 */
public class QuestionFragment extends Fragment implements IQuestion {

    TextView txt_question_text;
    CheckBox checkboxA, checkboxB, checkboxC, checkboxD;
    FrameLayout layout_image;
    ProgressBar progressBar;

    Question question;
    int questionIndex=-1;

    public QuestionFragment() {
        // Required empty public constructor
    }


    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View itemView = inflater.inflate(R.layout.fragment_question, container, false);

        //Get Q
        questionIndex = getArguments().getInt("index", -1);
        question = Common.questionList.get(questionIndex);

        if(question != null) {
            layout_image = (FrameLayout) itemView.findViewById(R.id.layout_image);
            progressBar = (ProgressBar)itemView.findViewById(R.id.progress_bar);
           if(question.isImageQuestion())
           {
               ImageView img_question = (ImageView)itemView.findViewById(R.id.img_question);
               Picasso.get().load(question.getQuestionImage()).into(img_question, new Callback() {
                   @Override
                   public void onSuccess() {
                       progressBar.setVisibility(View.GONE);
                   }

                   @Override
                   public void onError(Exception e) {
                       Toast.makeText(getContext(), ""+e.getMessage(), Toast.LENGTH_SHORT).show();
                   }
               });
           }
           else
               layout_image.setVisibility(View.GONE);

            //View
            txt_question_text = (TextView) itemView.findViewById(R.id.txt_question_text);
            txt_question_text.setText(question.getQuestionText());

            checkboxA = (CheckBox) itemView.findViewById(R.id.checkboxA);
            checkboxA.setText(question.getAnswerA());
            checkboxA.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
                @Override
                public void onCheckedChanged(CompoundButton compoundButton, boolean b) {
                    if(b)
                        Common.selected_values.add(checkboxA.getText().toString());
                    else
                        Common.selected_values.remove(checkboxA.getText().toString());
                }
            });

            checkboxB = (CheckBox) itemView.findViewById(R.id.checkboxB);
            checkboxB.setText(question.getAnswerB());
            checkboxB.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
                @Override
                public void onCheckedChanged(CompoundButton compoundButton, boolean b) {
                    if(b)
                        Common.selected_values.add(checkboxB.getText().toString());
                    else
                        Common.selected_values.remove(checkboxB.getText().toString());
                }
            });

            checkboxC = (CheckBox) itemView.findViewById(R.id.checkboxC);
            checkboxC.setText(question.getAnswerC());
            checkboxC.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
                @Override
                public void onCheckedChanged(CompoundButton compoundButton, boolean b) {
                    if(b)
                        Common.selected_values.add(checkboxC.getText().toString());
                    else
                        Common.selected_values.remove(checkboxC.getText().toString());
                }
            });

            checkboxD = (CheckBox) itemView.findViewById(R.id.checkboxD);
            checkboxD.setText(question.getAnswerD());
            checkboxD.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
                @Override
                public void onCheckedChanged(CompoundButton compoundButton, boolean b) {
                    if(b)
                        Common.selected_values.add(checkboxD.getText().toString());
                    else
                        Common.selected_values.remove(checkboxD.getText().toString());
                }
            });



        }

        return itemView;
    }

    @Override
    public CurrentQuestion getSelectedAnswer() {
        //this will return state of answer

        CurrentQuestion currentQuestion = new CurrentQuestion(questionIndex,Common.ANSWER_TYPE.NO_ANSWER); //DEFAULT
        StringBuilder result = new StringBuilder();
        if(Common.selected_values.size() > 1)
        {
            //If multichoice split answer to array
            Object[] arrayAnswer = Common.selected_values.toArray();
            for(int i=0; i<arrayAnswer.length;i++)
                if(i<arrayAnswer.length-1)
                    result.append(new StringBuilder(((String)arrayAnswer[i]).substring(0,1)).append(",")); //take first letter of answer
                else
                    result.append(new StringBuilder((String)arrayAnswer[i]).substring(0,1));
        }
        else if(Common.selected_values.size() == 1)
        {
            //If one choice
            Object[] arrayAnswer = Common.selected_values.toArray();
            result.append((String)arrayAnswer[0]).substring(0,1);
        }

        if(question != null)
        {
            //Compare user pick with correct answer
            if(!TextUtils.isEmpty(result)) {
                if (result.toString().equals(question.getCorrectAnswer()))
                    currentQuestion.setType(Common.ANSWER_TYPE.RIGHT_ANSWER);
                else
                    currentQuestion.setType(Common.ANSWER_TYPE.WRONG_ANSWER);
            }
            else
                currentQuestion.setType(Common.ANSWER_TYPE.NO_ANSWER);
        }
        else
        {
            Toast.makeText(getContext(), "Cannot get question, sorry Boss Man", Toast.LENGTH_SHORT).show();
            currentQuestion.setType(Common.ANSWER_TYPE.NO_ANSWER);
        }
        Common.selected_values.clear(); //clear selected_value when compare done
        return currentQuestion;
    }

    @Override
    public void showCorrectAnswer() {

        //bold correct answer
        String[] correctAnswer = question.getCorrectAnswer().split(",");
        for(String answer:correctAnswer)
        {
            if(answer.equals("A"))
            {
                checkboxA.setTypeface(null,Typeface.BOLD);
                checkboxA.setTextColor(Color.RED);
            }
            else if(answer.equals("B"))
            {
                checkboxB.setTypeface(null,Typeface.BOLD);
                checkboxB.setTextColor(Color.RED);
            }
            else if(answer.equals("C"))
            {
                checkboxC.setTypeface(null,Typeface.BOLD);
                checkboxC.setTextColor(Color.RED);
            }
            else if(answer.equals("D"))
            {
                checkboxD.setTypeface(null,Typeface.BOLD);
                checkboxD.setTextColor(Color.RED);
            }
        }

    }

    @Override
    public void disableAnswer() {
        checkboxA.setEnabled(false);
        checkboxB.setEnabled(false);
        checkboxC.setEnabled(false);
        checkboxD.setEnabled(false);

    }

    @Override
    public void resetQuestion() {
        checkboxA.setEnabled(true);
        checkboxB.setEnabled(true);
        checkboxC.setEnabled(true);
        checkboxD.setEnabled(true);

        checkboxA.setChecked(false);
        checkboxB.setChecked(false);
        checkboxC.setChecked(false);
        checkboxD.setChecked(false);

        //remove bold on text
        checkboxA.setTypeface(null, Typeface.NORMAL);
        checkboxA.setTextColor(Color.BLACK);
        checkboxB.setTypeface(null, Typeface.NORMAL);
        checkboxB.setTextColor(Color.BLACK);
        checkboxC.setTypeface(null, Typeface.NORMAL);
        checkboxC.setTextColor(Color.BLACK);
        checkboxD.setTypeface(null, Typeface.NORMAL);
        checkboxD.setTextColor(Color.BLACK);

    }
}
